@@ -27,7 +27,7 @@ class _TestBase:
         self.loop.close()
 
         # operation blocked when the loop is closed
-        f = asyncio.Future(loop=self.loop)
+        f = asyncio.Future()
         self.assertRaises(RuntimeError, self.loop.run_forever)
         self.assertRaises(RuntimeError, self.loop.run_until_complete, f)
 
@@ -36,7 +36,7 @@ class _TestBase:
         h = self.loop.call_soon(lambda: None)
         wd['h'] = h  # Would fail without __weakref__ slot.
 
-    def test_call_soon(self):
+    def test_call_soon_1(self):
         calls = []
 
         def cb(inc):
@@ -56,6 +56,22 @@ class _TestBase:
 
         self.assertEqual(calls, [10, 1])
 
+    def test_call_soon_2(self):
+        waiter = self.loop.create_future()
+        waiter_r = weakref.ref(waiter)
+        self.loop.call_soon(lambda f: f.set_result(None), waiter)
+        self.loop.run_until_complete(waiter)
+        del waiter
+        self.assertIsNone(waiter_r())
+
+    def test_call_soon_3(self):
+        waiter = self.loop.create_future()
+        waiter_r = weakref.ref(waiter)
+        self.loop.call_soon(lambda f=waiter: f.set_result(None))
+        self.loop.run_until_complete(waiter)
+        del waiter
+        self.assertIsNone(waiter_r())
+
     def test_call_soon_base_exc(self):
         def cb():
             raise KeyboardInterrupt()
@@ -83,7 +99,7 @@ class _TestBase:
 
             meth(cb)
             self.assertIsNone(context)
-            self.loop.run_until_complete(asyncio.sleep(0.05, loop=self.loop))
+            self.loop.run_until_complete(asyncio.sleep(0.05))
 
             self.assertIs(type(context['exception']), ZeroDivisionError)
             self.assertTrue(context['message'].startswith(
@@ -151,15 +167,33 @@ class _TestBase:
         # libuv cached time.
 
         async def main():
-            await asyncio.sleep(0.001, loop=self.loop)
+            await asyncio.sleep(0.001)
             time.sleep(0.01)
-            await asyncio.sleep(0.01, loop=self.loop)
+            await asyncio.sleep(0.01)
 
         started = time.monotonic()
         self.loop.run_until_complete(main())
         delta = time.monotonic() - started
         self.assertGreater(delta, 0.019)
 
+    def test_call_later_3(self):
+        # a memory leak regression test
+        waiter = self.loop.create_future()
+        waiter_r = weakref.ref(waiter)
+        self.loop.call_later(0.01, lambda f: f.set_result(None), waiter)
+        self.loop.run_until_complete(waiter)
+        del waiter
+        self.assertIsNone(waiter_r())
+
+    def test_call_later_4(self):
+        # a memory leak regression test
+        waiter = self.loop.create_future()
+        waiter_r = weakref.ref(waiter)
+        self.loop.call_later(0.01, lambda f=waiter: f.set_result(None))
+        self.loop.run_until_complete(waiter)
+        del waiter
+        self.assertIsNone(waiter_r())
+
     def test_call_later_negative(self):
         calls = []
 
@@ -171,6 +205,19 @@ class _TestBase:
         self.loop.run_forever()
         self.assertEqual(calls, ['a'])
 
+    def test_call_later_rounding(self):
+        # Refs #233, call_later() and call_at() shouldn't call cb early
+
+        def cb():
+            self.loop.stop()
+
+        for i in range(8):
+            self.loop.call_later(0.06 + 0.01, cb)  # 0.06999999999999999
+            started = int(round(self.loop.time() * 1000))
+            self.loop.run_forever()
+            finished = int(round(self.loop.time() * 1000))
+            self.assertGreaterEqual(finished - started, 69)
+
     def test_call_at(self):
         if os.environ.get('TRAVIS_OS_NAME'):
             # Time seems to be really unpredictable on Travis.
@@ -284,7 +331,7 @@ class _TestBase:
             TypeError, self.loop.run_until_complete, 'blah')
 
     def test_run_until_complete_loop(self):
-        task = asyncio.Future(loop=self.loop)
+        task = asyncio.Future()
         other_loop = self.new_loop()
         self.addCleanup(other_loop.close)
         self.assertRaises(
@@ -300,19 +347,16 @@ class _TestBase:
         if self.implementation == 'asyncio' and sys.version_info < (3, 6, 2):
             raise unittest.SkipTest('unfixed asyncio')
 
-        class ShowStopper(BaseException):
-            pass
-
         async def foo(delay):
-            await asyncio.sleep(delay, loop=self.loop)
+            await asyncio.sleep(delay)
 
         def throw():
-            raise ShowStopper
+            raise KeyboardInterrupt
 
         self.loop.call_soon(throw)
         try:
             self.loop.run_until_complete(foo(0.1))
-        except ShowStopper:
+        except KeyboardInterrupt:
             pass
 
         # This call fails if run_until_complete does not clean up
@@ -326,7 +370,7 @@ class _TestBase:
         self.loop.call_soon(lambda: time.sleep(0.3))
 
         with mock.patch.object(logger, 'warning') as log:
-            self.loop.run_until_complete(asyncio.sleep(0, loop=self.loop))
+            self.loop.run_until_complete(asyncio.sleep(0))
 
         self.assertEqual(log.call_count, 1)
         # format message
@@ -342,7 +386,7 @@ class _TestBase:
         self.loop.call_later(0.01, lambda: time.sleep(0.3))
 
         with mock.patch.object(logger, 'warning') as log:
-            self.loop.run_until_complete(asyncio.sleep(0.02, loop=self.loop))
+            self.loop.run_until_complete(asyncio.sleep(0.02))
 
         self.assertEqual(log.call_count, 1)
         # format message
@@ -382,7 +426,7 @@ class _TestBase:
 
         # Test call_soon (events.Handle)
         with mock.patch.object(logger, 'error') as log:
-            fut = asyncio.Future(loop=self.loop)
+            fut = asyncio.Future()
             self.loop.call_soon(zero_error, fut)
             fut.add_done_callback(lambda fut: self.loop.stop())
             self.loop.run_forever()
@@ -392,7 +436,7 @@ class _TestBase:
 
         # Test call_later (events.TimerHandle)
         with mock.patch.object(logger, 'error') as log:
-            fut = asyncio.Future(loop=self.loop)
+            fut = asyncio.Future()
             self.loop.call_later(0.01, zero_error, fut)
             fut.add_done_callback(lambda fut: self.loop.stop())
             self.loop.run_forever()
@@ -476,8 +520,7 @@ class _TestBase:
         class MyTask(asyncio.Task):
             pass
 
-        @asyncio.coroutine
-        def coro():
+        async def coro():
             pass
 
         factory = lambda loop, coro: MyTask(coro, loop=loop)
@@ -515,31 +558,34 @@ class _TestBase:
             raise unittest.SkipTest()
 
         waiter = self._compile_agen(
-            '''async def waiter(timeout, finalized, loop):
+            '''async def waiter(timeout, finalized):
                 try:
-                    await asyncio.sleep(timeout, loop=loop)
+                    await asyncio.sleep(timeout)
                     yield 1
                 finally:
-                    await asyncio.sleep(0, loop=loop)
+                    await asyncio.sleep(0)
                     finalized.append(1)
             ''')
 
         async def wait():
-            async for _ in waiter(1, finalized, self.loop):
+            async for _ in waiter(1, finalized):
                 pass
 
         t1 = self.loop.create_task(wait())
         t2 = self.loop.create_task(wait())
 
-        self.loop.run_until_complete(asyncio.sleep(0.1, loop=self.loop))
+        self.loop.run_until_complete(asyncio.sleep(0.1))
 
+        t1.cancel()
+        t2.cancel()
         self.loop.run_until_complete(self.loop.shutdown_asyncgens())
         self.assertEqual(finalized, [1, 1])
 
-        # Silence warnings
-        t1.cancel()
-        t2.cancel()
-        self.loop.run_until_complete(asyncio.sleep(0.1, loop=self.loop))
+        for t in {t1, t2}:
+            try:
+                self.loop.run_until_complete(t)
+            except asyncio.CancelledError:
+                pass
 
     def test_shutdown_asyncgens_02(self):
         if not hasattr(self.loop, 'shutdown_asyncgens'):
@@ -554,20 +600,20 @@ class _TestBase:
             if expected in context['message']:
                 logged += 1
 
-        waiter = self._compile_agen('''async def waiter(timeout, loop):
+        waiter = self._compile_agen('''async def waiter(timeout):
             try:
-                await asyncio.sleep(timeout, loop=loop)
+                await asyncio.sleep(timeout)
                 yield 1
             finally:
                 1 / 0
         ''')
 
         async def wait():
-            async for _ in waiter(1, self.loop):
+            async for _ in waiter(1):
                 pass
 
         t = self.loop.create_task(wait())
-        self.loop.run_until_complete(asyncio.sleep(0.1, loop=self.loop))
+        self.loop.run_until_complete(asyncio.sleep(0.1))
 
         self.loop.set_exception_handler(logger)
         self.loop.run_until_complete(self.loop.shutdown_asyncgens())
@@ -576,7 +622,7 @@ class _TestBase:
 
         # Silence warnings
         t.cancel()
-        self.loop.run_until_complete(asyncio.sleep(0.1, loop=self.loop))
+        self.loop.run_until_complete(asyncio.sleep(0.1))
 
     def test_shutdown_asyncgens_03(self):
         if not hasattr(self.loop, 'shutdown_asyncgens'):
@@ -593,14 +639,14 @@ class _TestBase:
             await waiter().asend(None)
 
         self.loop.run_until_complete(foo())
-        self.loop.run_until_complete(asyncio.sleep(0.01, loop=self.loop))
+        self.loop.run_until_complete(asyncio.sleep(0.01))
 
     def test_inf_wait_for(self):
         async def foo():
-            await asyncio.sleep(0.1, loop=self.loop)
+            await asyncio.sleep(0.1)
             return 123
         res = self.loop.run_until_complete(
-            asyncio.wait_for(foo(), timeout=float('inf'), loop=self.loop))
+            asyncio.wait_for(foo(), timeout=float('inf')))
         self.assertEqual(res, 123)
 
 
@@ -694,6 +740,37 @@ class TestBaseUV(_TestBase, UVTestCase):
             self.assertIs(type(_context['context']['exception']),
                           ZeroDivisionError)
 
+    def test_big_call_later_timeout(self):
+        OK, NOT_OK = 0, 0
+
+        async def sleep(delay_name, delay):
+            nonlocal OK, NOT_OK
+            try:
+                await asyncio.sleep(delay)
+            except asyncio.CancelledError:
+                OK += 1
+            except Exception:
+                NOT_OK += 1
+
+        async def main():
+            tests = [
+                sleep("infinity", float("inf")),
+                sleep("sys.maxsize", float(sys.maxsize)),
+                sleep("sys.maxsize", sys.maxsize),
+                sleep("2**55", 2**55),
+                sleep("2**54", 2**54),
+            ]
+            tasks = [self.loop.create_task(test) for test in tests]
+            await asyncio.sleep(0.1)
+            for task in tasks:
+                task.cancel()
+                await task
+
+        self.loop.run_until_complete(main())
+
+        self.assertEqual(OK, 5)
+        self.assertEqual(NOT_OK, 0)
+
 
 class TestBaseAIO(_TestBase, AIOTestCase):
     pass
@@ -715,8 +792,7 @@ class TestPolicy(unittest.TestCase):
     @unittest.skipUnless(hasattr(asyncio, '_get_running_loop'),
                          'No asyncio._get_running_loop')
     def test_running_loop_within_a_loop(self):
-        @asyncio.coroutine
-        def runner(loop):
+        async def runner(loop):
             loop.run_forever()
 
         try:
